@@ -361,13 +361,6 @@ void Generator::PrintTopBoilerplate() const {
                     "protobuf_python_version", PROTOBUF_PYTHON_VERSION_STRING);
   }
   printer_->Print("\"\"\"Generated protocol buffer code.\"\"\"\n");
-  if (!opensource_runtime_) {
-    // This import is needed so that compatibility proto1 compiler output
-    // inserted at protoc_insertion_point can refer to other protos like
-    // google3.a.b.c. Code generated by proto2 compiler doesn't do it, and
-    // instead uses aliases assigned when importing modules.
-    printer_->Print("import google3\n");
-  }
   bool runtime_version_disabled = false;
   printer_->Print(
       "from google.protobuf import descriptor as _descriptor\n"
@@ -466,8 +459,11 @@ std::string Generator::GetResolvedFeatures(
   }
 
   // Load the resolved features from our pool.
-  const Descriptor* feature_set = file_->pool()->FindMessageTypeByName(
-      FeatureSet::GetDescriptor()->full_name());
+  const Descriptor* feature_set =
+      file_->FindMessageTypeByName(FeatureSet::GetDescriptor()->name());
+  ABSL_CHECK(feature_set != nullptr)
+      << "Malformed descriptor.proto doesn't contain "
+      << FeatureSet::GetDescriptor()->full_name();
   auto message_factory = absl::make_unique<DynamicMessageFactory>();
   auto features =
       absl::WrapUnique(message_factory->GetPrototype(feature_set)->New());
@@ -483,8 +479,17 @@ std::string Generator::GetResolvedFeatures(
     // Assume these are all enums.  If we add non-enum global features or any
     // python-specific features, we will need to come back and improve this
     // logic.
-    ABSL_CHECK(field->enum_type() != nullptr)
-        << "Unexpected non-enum field found!";
+    if (field->type() != FieldDescriptor::TYPE_ENUM) {
+      ABSL_CHECK(field->is_extension())
+          << "Unsupported non-enum global feature found: "
+          << field->full_name();
+      // Placeholder for python-specific features.
+      ABSL_CHECK(field->number() != 1003)
+          << "Unsupported python-specific feature found: "
+          << field->full_name();
+      // Skip any non-python language-specific features.
+      continue;
+    }
     if (field->options().retention() == FieldOptions::RETENTION_SOURCE) {
       // Skip any source-retention features.
       continue;
@@ -566,8 +571,8 @@ void Generator::PrintResolvedFeatures() const {
 void Generator::PrintFileDescriptor() const {
   absl::flat_hash_map<absl::string_view, std::string> m;
   m["descriptor_name"] = kDescriptorKey;
-  m["name"] = file_->name();
-  m["package"] = file_->package();
+  m["name"] = std::string(file_->name());
+  m["package"] = std::string(file_->package());
   m["syntax"] = GetLegacySyntaxName(GetEdition(*file_));
   m["edition"] = Edition_Name(GetEdition(*file_));
   m["options"] = OptionsValue(proto_.options().SerializeAsString());
@@ -649,8 +654,8 @@ void Generator::PrintEnum(const EnumDescriptor& enum_descriptor,
   std::string module_level_descriptor_name =
       ModuleLevelDescriptorName(enum_descriptor);
   m["descriptor_name"] = module_level_descriptor_name;
-  m["name"] = enum_descriptor.name();
-  m["full_name"] = enum_descriptor.full_name();
+  m["name"] = std::string(enum_descriptor.name());
+  m["full_name"] = std::string(enum_descriptor.full_name());
   m["file"] = kDescriptorKey;
   const char enum_descriptor_template[] =
       "$descriptor_name$ = _descriptor.EnumDescriptor(\n"
@@ -705,6 +710,7 @@ void Generator::PrintMessageDescriptors() const {
   }
 }
 
+// TODO: Remove python service code from opensource.
 void Generator::PrintServiceDescriptors() const {
   for (int i = 0; i < file_->service_count(); ++i) {
     PrintServiceDescriptor(*file_->service(i));
@@ -723,7 +729,7 @@ void Generator::PrintServiceDescriptor(
     const ServiceDescriptor& descriptor) const {
   absl::flat_hash_map<absl::string_view, std::string> m;
   m["service_name"] = ModuleLevelServiceDescriptorName(descriptor);
-  m["name"] = descriptor.name();
+  m["name"] = std::string(descriptor.name());
   m["file"] = kDescriptorKey;
   printer_->Print(m, "$service_name$ = $file$.services_by_name['$name$']\n");
 }
@@ -773,8 +779,8 @@ void Generator::PrintServiceStub(const ServiceDescriptor& descriptor) const {
 void Generator::PrintDescriptor(const Descriptor& message_descriptor,
                                 const DescriptorProto& proto) const {
   absl::flat_hash_map<absl::string_view, std::string> m;
-  m["name"] = message_descriptor.name();
-  m["full_name"] = message_descriptor.full_name();
+  m["name"] = std::string(message_descriptor.name());
+  m["full_name"] = std::string(message_descriptor.full_name());
   m["file"] = kDescriptorKey;
 
   PrintNestedDescriptors(message_descriptor, proto);
@@ -839,8 +845,8 @@ void Generator::PrintDescriptor(const Descriptor& message_descriptor,
   for (int i = 0; i < message_descriptor.oneof_decl_count(); ++i) {
     const OneofDescriptor* desc = message_descriptor.oneof_decl(i);
     m.clear();
-    m["name"] = desc->name();
-    m["full_name"] = desc->full_name();
+    m["name"] = std::string(desc->name());
+    m["full_name"] = std::string(desc->full_name());
     m["index"] = absl::StrCat(desc->index());
     options_string =
         OptionsValue(proto.oneof_decl(i).options().SerializeAsString());
@@ -983,9 +989,9 @@ void Generator::FixForeignFieldsInDescriptor(
     absl::flat_hash_map<absl::string_view, std::string> m;
     const OneofDescriptor* oneof = descriptor.oneof_decl(i);
     m["descriptor_name"] = ModuleLevelDescriptorName(descriptor);
-    m["oneof_name"] = oneof->name();
+    m["oneof_name"] = std::string(oneof->name());
     for (int j = 0; j < oneof->field_count(); ++j) {
-      m["field_name"] = oneof->field(j)->name();
+      m["field_name"] = std::string(oneof->field(j)->name());
       printer_->Print(
           m,
           "$descriptor_name$.oneofs_by_name['$oneof_name$'].fields.append(\n"
@@ -1001,7 +1007,7 @@ void Generator::FixForeignFieldsInDescriptor(
 void Generator::AddMessageToFileDescriptor(const Descriptor& descriptor) const {
   absl::flat_hash_map<absl::string_view, std::string> m;
   m["descriptor_name"] = kDescriptorKey;
-  m["message_name"] = descriptor.name();
+  m["message_name"] = std::string(descriptor.name());
   m["message_descriptor_name"] = ModuleLevelDescriptorName(descriptor);
   const char file_descriptor_template[] =
       "$descriptor_name$.message_types_by_name['$message_name$'] = "
@@ -1013,7 +1019,7 @@ void Generator::AddServiceToFileDescriptor(
     const ServiceDescriptor& descriptor) const {
   absl::flat_hash_map<absl::string_view, std::string> m;
   m["descriptor_name"] = kDescriptorKey;
-  m["service_name"] = descriptor.name();
+  m["service_name"] = std::string(descriptor.name());
   m["service_descriptor_name"] = ModuleLevelServiceDescriptorName(descriptor);
   const char file_descriptor_template[] =
       "$descriptor_name$.services_by_name['$service_name$'] = "
@@ -1025,7 +1031,7 @@ void Generator::AddEnumToFileDescriptor(
     const EnumDescriptor& descriptor) const {
   absl::flat_hash_map<absl::string_view, std::string> m;
   m["descriptor_name"] = kDescriptorKey;
-  m["enum_name"] = descriptor.name();
+  m["enum_name"] = std::string(descriptor.name());
   m["enum_descriptor_name"] = ModuleLevelDescriptorName(descriptor);
   const char file_descriptor_template[] =
       "$descriptor_name$.enum_types_by_name['$enum_name$'] = "
@@ -1037,7 +1043,7 @@ void Generator::AddExtensionToFileDescriptor(
     const FieldDescriptor& descriptor) const {
   absl::flat_hash_map<absl::string_view, std::string> m;
   m["descriptor_name"] = kDescriptorKey;
-  m["field_name"] = descriptor.name();
+  m["field_name"] = std::string(descriptor.name());
   m["resolved_name"] = ResolveKeyword(descriptor.name());
   const char file_descriptor_template[] =
       "$descriptor_name$.extensions_by_name['$field_name$'] = "
@@ -1146,7 +1152,7 @@ void Generator::PrintEnumValueDescriptor(
   std::string options_string;
   proto.options().SerializeToString(&options_string);
   absl::flat_hash_map<absl::string_view, std::string> m;
-  m["name"] = descriptor.name();
+  m["name"] = std::string(descriptor.name());
   m["index"] = absl::StrCat(descriptor.index());
   m["number"] = absl::StrCat(descriptor.number());
   m["options"] = OptionsValue(options_string);
@@ -1164,8 +1170,8 @@ void Generator::PrintFieldDescriptor(const FieldDescriptor& field,
   std::string options_string;
   proto.options().SerializeToString(&options_string);
   absl::flat_hash_map<absl::string_view, std::string> m;
-  m["name"] = field.name();
-  m["full_name"] = field.full_name();
+  m["name"] = std::string(field.name());
+  m["full_name"] = std::string(field.full_name());
   m["index"] = absl::StrCat(field.index());
   m["number"] = absl::StrCat(field.number());
   m["type"] = absl::StrCat(field.type());
@@ -1430,7 +1436,7 @@ void Generator::FixOptionsForEnum(const EnumDescriptor& enum_descriptor,
     PrintDescriptorOptionsFixingCode(
         value_descriptor, proto.value(i),
         absl::StrFormat("%s.values_by_name[\"%s\"]", descriptor_name.c_str(),
-                        value_descriptor.name().c_str()));
+                        value_descriptor.name()));
   }
 }
 
@@ -1459,7 +1465,7 @@ void Generator::FixOptionsForField(const FieldDescriptor& field,
   if (field.is_extension()) {
     if (field.extension_scope() == nullptr) {
       // Top level extensions.
-      field_name = field.name();
+      field_name = std::string(field.name());
     } else {
       field_name = FieldReferencingExpression(field.extension_scope(), field,
                                               "extensions_by_name");
